@@ -22,8 +22,8 @@ from brownie.exceptions import (
 )
 from brownie.utils import color
 
-from .rpc import Rpc, _revert_register
-from .state import TxHistory
+from .rpc import Rpc
+from .state import Chain, TxHistory, _revert_register
 from .transaction import TransactionReceipt
 from .web3 import _resolve_address, web3
 
@@ -90,12 +90,14 @@ class Accounts(metaclass=_Singleton):
             return False
 
     def __repr__(self) -> str:
-        return str(self._accounts)
+        if CONFIG.argv["cli"] == "console":
+            return str(self._accounts)
+        return super().__repr__()
 
     def __iter__(self) -> Iterator:
         return iter(self._accounts)
 
-    def __getitem__(self, key: int) -> Any:
+    def __getitem__(self, key: int) -> "_PrivateKeyAccount":
         return self._accounts[key]
 
     def __delitem__(self, key: int) -> None:
@@ -351,7 +353,7 @@ class _PrivateKeyAccount(PublicKeyAccount):
         gas_price: Optional[int] = None,
         nonce: Optional[int] = None,
         required_confs: int = 1,
-        silent: bool = False,
+        silent: bool = None,
     ) -> Any:
         """Deploys a contract.
 
@@ -377,6 +379,8 @@ class _PrivateKeyAccount(PublicKeyAccount):
                 f"Local RPC using '{rpc.evm_version()}' but contract was compiled for '{evm}'"
             )
         data = contract.deploy.encode_input(*args)
+        if silent is None:
+            silent = bool(CONFIG.mode == "test" or CONFIG.argv["silent"])
         with self._lock:
             try:
                 txid = self._transact(  # type: ignore
@@ -411,7 +415,7 @@ class _PrivateKeyAccount(PublicKeyAccount):
 
         if rpc.is_active():
             undo_thread = threading.Thread(
-                target=rpc._add_to_undo_buffer,
+                target=Chain()._add_to_undo_buffer,
                 args=(
                     receipt,
                     self.deploy,
@@ -471,8 +475,8 @@ class _PrivateKeyAccount(PublicKeyAccount):
         data: str = None,
         nonce: Optional[int] = None,
         required_confs: int = 1,
-        silent: bool = False,
-    ) -> "TransactionReceipt":
+        silent: bool = None,
+    ) -> TransactionReceipt:
         """
         Broadcast a transaction from this account.
 
@@ -488,6 +492,8 @@ class _PrivateKeyAccount(PublicKeyAccount):
         Returns:
             TransactionReceipt object
         """
+        if silent is None:
+            silent = bool(CONFIG.mode == "test" or CONFIG.argv["silent"])
         with self._lock:
             tx = {
                 "from": self.address,
@@ -515,7 +521,7 @@ class _PrivateKeyAccount(PublicKeyAccount):
             history._add_tx(receipt)
         if rpc.is_active():
             undo_thread = threading.Thread(
-                target=rpc._add_to_undo_buffer,
+                target=Chain()._add_to_undo_buffer,
                 args=(receipt, self.transfer, (to, amount, gas_limit, gas_price, data, None), {}),
                 daemon=True,
             )
