@@ -3,7 +3,6 @@
 import json
 import os
 import re
-import sys
 import warnings
 from pathlib import Path
 from textwrap import TextWrapper
@@ -16,6 +15,8 @@ import solcx
 from eth_utils import remove_0x_prefix
 from hexbytes import HexBytes
 from semantic_version import Version
+from vvm import get_installable_vyper_versions
+from vvm.utils.convert import to_vyper_version
 
 from brownie._config import CONFIG, REQUEST_HEADERS
 from brownie.convert.datatypes import Wei
@@ -37,10 +38,16 @@ from brownie.project import compiler, ethpm
 from brownie.typing import AccountsType, TransactionReceiptType
 from brownie.utils import color
 
-from . import accounts, rpc
+from . import accounts, chain
 from .event import _add_deployment_topics, _get_topics
-from .rpc import _revert_register
-from .state import _add_contract, _add_deployment, _find_contract, _get_deployment, _remove_contract
+from .state import (
+    _add_contract,
+    _add_deployment,
+    _find_contract,
+    _get_deployment,
+    _remove_contract,
+    _revert_register,
+)
 from .web3 import _resolve_address, web3
 
 _unverified_addresses: Set = set()
@@ -116,7 +123,9 @@ class ContractContainer(_ContractBase):
         return len(self._contracts)
 
     def __repr__(self) -> str:
-        return str(self._contracts)
+        if CONFIG.argv["cli"] == "console":
+            return str(self._contracts)
+        return super().__repr__()
 
     def _reset(self) -> None:
         for contract in self._contracts:
@@ -335,7 +344,13 @@ class _DeployedContractBase(_ContractBase):
             getattr(self, abi["name"])._add_fn(abi, natspec)
 
     def _check_and_set(self, name: str, obj: Any) -> None:
-        if hasattr(self, name):
+        if name == "balance":
+            warnings.warn(
+                f"'{self._name}' defines a 'balance' function, "
+                f"'{self._name}.balance' is unavailable",
+                BrownieEnvironmentWarning,
+            )
+        elif hasattr(self, name):
             raise AttributeError(f"Namespace collision: '{self._name}.{name}'")
         setattr(self, name, obj)
 
@@ -656,20 +671,28 @@ class Contract(_DeployedContractBase):
         if not is_verified:
             return cls.from_abi(name, address, abi, owner)
 
-        try:
-            version = Version(data["result"][0]["CompilerVersion"].lstrip("v")).truncate()
-        except Exception:
-            version = Version("0.0.0")
-        if version < Version("0.4.22") or (
-            # special case for OSX because installing 0.4.x versions is problematic
-            sys.platform == "darwin"
-            and version < Version("0.5.0")
-            and f"v{version}" not in solcx.get_installed_solc_versions()
-        ):
+        compiler_str = data["result"][0]["CompilerVersion"]
+        if compiler_str.startswith("vyper:"):
+            try:
+                version = to_vyper_version(compiler_str[6:])
+                is_compilable = version in get_installable_vyper_versions()
+            except Exception:
+                is_compilable = False
+        else:
+            try:
+                version = Version(compiler_str.lstrip("v")).truncate()
+                is_compilable = (
+                    version
+                    in solcx.get_installable_solc_versions() + solcx.get_installed_solc_versions()
+                )
+            except Exception:
+                is_compilable = False
+
+        if not is_compilable:
             if not silent:
                 warnings.warn(
-                    f"{address}: target compiler '{data['result'][0]['CompilerVersion']}' is "
-                    "unsupported by Brownie. Some functionality will not be available.",
+                    f"{address}: target compiler '{compiler_str}' cannot be installed or is not "
+                    "supported by Brownie. Some debugging functionality will not be available.",
                     BrownieCompilerWarning,
                 )
             return cls.from_abi(name, address, abi, owner)
@@ -682,9 +705,10 @@ class Contract(_DeployedContractBase):
         if evm_version == "Default":
             evm_version = None
 
-        if data["result"][0]["SourceCode"].startswith("{"):
+        source_str = "\n".join(data["result"][0]["SourceCode"].splitlines())
+        if source_str.startswith("{"):
             # source was verified using compiler standard JSON
-            input_json = json.loads(data["result"][0]["SourceCode"][1:-1])
+            input_json = json.loads(source_str[1:-1])
             sources = {k: v["content"] for k, v in input_json["sources"].items()}
             evm_version = input_json["settings"].get("evmVersion", evm_version)
 
@@ -695,10 +719,19 @@ class Contract(_DeployedContractBase):
             output_json = compiler.compile_from_input_json(input_json)
             build_json = compiler.generate_build_json(input_json, output_json)
         else:
-            # source was submitted as a single flattened file
-            sources = {f"{name}-flattened.sol": data["result"][0]["SourceCode"]}
+            # source was submitted as a single file
+            if compiler_str.startswith("vyper"):
+                path_str = f"{name}.vy"
+            else:
+                path_str = f"{name}-flattened.sol"
+
+            sources = {path_str: source_str}
             build_json = compiler.compile_and_format(
-                sources, solc_version=str(version), optimizer=optimizer, evm_version=evm_version
+                sources,
+                solc_version=str(version),
+                vyper_version=str(version),
+                optimizer=optimizer,
+                evm_version=evm_version,
             )
 
         build_json = build_json[name]
@@ -892,6 +925,7 @@ class _ContractMethod:
         self.abi = abi
         self._owner = owner
         self.signature = build_function_selector(abi)
+        self._input_sig = build_function_signature(abi)
         self.natspec = natspec or {}
 
     def __repr__(self) -> str:
@@ -986,6 +1020,23 @@ class _ContractMethod:
             data=self.encode_input(*args),
         )
 
+    def decode_input(self, hexstr: str) -> List:
+        """
+        Decode input call data for this method.
+
+        Arguments
+        ---------
+        hexstr : str
+            Hexstring of input call data
+
+        Returns
+        -------
+        Decoded values
+        """
+        types_list = get_type_strings(self.abi["inputs"])
+        result = eth_abi.decode_abi(types_list, HexBytes(hexstr)[4:])
+        return format_input(self.abi, result)
+
     def encode_input(self, *args: Tuple) -> str:
         """
         Generate encoded ABI data to call the method with the given arguments.
@@ -1005,12 +1056,18 @@ class _ContractMethod:
         return self.signature + eth_abi.encode_abi(types_list, data).hex()
 
     def decode_output(self, hexstr: str) -> Tuple:
-        """Decodes hexstring data returned by this method.
+        """
+        Decode hexstring data returned by this method.
 
-        Args:
-            hexstr: Hexstring of returned call data
+        Arguments
+        ---------
+        hexstr : str
+            Hexstring of returned call data
 
-        Returns: Decoded values."""
+        Returns
+        -------
+        Decoded values
+        """
         types_list = get_type_strings(self.abi["outputs"])
         result = eth_abi.decode_abi(types_list, HexBytes(hexstr))
         result = format_output(self.abi, result)
@@ -1091,12 +1148,13 @@ class ContractCall(_ContractMethod):
 
         try:
             self.transact(*args, tx)
+            chain.undo()
         except VirtualMachineError as exc:
             pc, revert_msg = exc.pc, exc.revert_msg
+            chain.undo()
         except Exception:
             pass
 
-        rpc.undo()
         try:
             return self.call(*args)
         except VirtualMachineError as exc:
