@@ -3,7 +3,7 @@
 import logging
 from collections import deque
 from hashlib import sha1
-from typing import Any, Dict, List, Optional, Set, Tuple
+from typing import Any, Dict, List, Optional, Set, Tuple, Union
 
 import solcast
 import solcx
@@ -61,33 +61,34 @@ def compile_from_input_json(
             print(f"  EVM Version: {input_json['settings']['evmVersion'].capitalize()}")
 
     try:
-        return solcx.compile_standard(
-            input_json,
-            optimize=optimizer["enabled"],
-            optimize_runs=optimizer["runs"],
-            evm_version=input_json["settings"]["evmVersion"],
-            allow_paths=allow_paths,
-        )
+        return solcx.compile_standard(input_json, allow_paths=allow_paths,)
     except solcx.exceptions.SolcError as e:
-        raise CompilerError(e)
+        raise CompilerError(e, "solc")
 
 
-def set_solc_version(version: str) -> str:
+def set_solc_version(version: Union[str, Version]) -> str:
     """Sets the solc version. If not available it will be installed."""
-    if Version(version.lstrip("v")) < Version("0.4.22"):
+    if not isinstance(version, Version):
+        version = Version(version.lstrip("v"))
+    if version < Version("0.4.22"):
         raise IncompatibleSolcVersion("Brownie only supports Solidity versions >=0.4.22")
     try:
         solcx.set_solc_version(version, silent=True)
     except solcx.exceptions.SolcNotInstalled:
+        if version not in _get_solc_version_list()[0]:
+            raise IncompatibleSolcVersion(
+                f"Cannot install Solidity v{version} on this OS. You may be able to "
+                f"manually compile from source with `solcx.compile_solc('{version}')`"
+            )
         install_solc(version)
         solcx.set_solc_version(version, silent=True)
     return str(solcx.get_solc_version())
 
 
-def install_solc(*versions: str) -> None:
+def install_solc(*versions: Union[Version, str]) -> None:
     """Installs solc versions."""
     for version in versions:
-        solcx.install_solc(str(version), show_progress=True)
+        solcx.install_solc(version, show_progress=True)
 
 
 def get_abi(contract_source: str, allow_paths: Optional[str] = None) -> Dict:
@@ -156,7 +157,7 @@ def find_solc_versions(
     # install new versions if needed
     if to_install:
         install_solc(*to_install)
-        installed_versions = [Version(i[1:]) for i in solcx.get_installed_solc_versions()]
+        installed_versions = solcx.get_installed_solc_versions()
     elif new_versions and not silent:
         print(
             f"New compatible solc version{'s' if len(new_versions) > 1 else ''}"
@@ -219,10 +220,10 @@ def find_best_solc_version(
 
 def _get_solc_version_list() -> Tuple[List, List]:
     global AVAILABLE_SOLC_VERSIONS
-    installed_versions = [Version(i[1:]) for i in solcx.get_installed_solc_versions()]
+    installed_versions = solcx.get_installed_solc_versions()
     if AVAILABLE_SOLC_VERSIONS is None:
         try:
-            AVAILABLE_SOLC_VERSIONS = [Version(i[1:]) for i in solcx.get_available_solc_versions()]
+            AVAILABLE_SOLC_VERSIONS = solcx.get_installable_solc_versions()
         except ConnectionError:
             if not installed_versions:
                 raise ConnectionError("Solc not installed and cannot connect to GitHub")
@@ -239,6 +240,9 @@ def _get_unique_build_json(
     }
 
     bytecode = _format_link_references(output_evm)
+    without_metadata = _remove_metadata(output_evm["deployedBytecode"]["object"])
+    instruction_count = len(without_metadata) // 2
+
     pc_map, statement_map, branch_map = _generate_coverage_data(
         output_evm["deployedBytecode"]["sourceMap"],
         output_evm["deployedBytecode"]["opcodes"],
@@ -246,13 +250,16 @@ def _get_unique_build_json(
         stmt_nodes,
         branch_nodes,
         has_fallback,
+        instruction_count,
     )
     return {
         "allSourcePaths": paths,
         "bytecode": bytecode,
         "bytecodeSha1": sha1(_remove_metadata(bytecode).encode()).hexdigest(),
         "coverageMap": {"statements": statement_map, "branches": branch_map},
-        "dependencies": [i.name for i in contract_node.dependencies],
+        "dependencies": [
+            i.name for i in contract_node.dependencies if i.nodeType == "ContractDefinition"
+        ],
         "offset": contract_node.offset,
         "pcMap": pc_map,
         "type": contract_node.contractKind,
@@ -284,6 +291,7 @@ def _generate_coverage_data(
     stmt_nodes: Dict,
     branch_nodes: Dict,
     has_fallback: bool,
+    instruction_count: int,
 ) -> Tuple:
     # Generates data used by Brownie for debugging and coverage evaluation
     if not opcodes_str:
@@ -402,7 +410,7 @@ def _generate_coverage_data(
             key = (pc_list[-1]["path"], pc_list[-1]["offset"])
             revert_map.setdefault(key, []).append(len(pc_list))
 
-    while opcodes and opcodes[0] not in ("INVALID", "STOP"):
+    while opcodes[0] not in ("INVALID", "STOP") and pc < instruction_count:
         # necessary because sometimes solidity returns an incomplete source map
         pc_list.append({"op": opcodes.popleft(), "pc": pc})
         pc += 1
