@@ -7,6 +7,7 @@ from pathlib import Path
 from typing import Dict, List, Optional, Tuple
 
 from semantic_version import NpmSpec
+from vvm.utils.convert import to_vyper_version
 
 from brownie.exceptions import NamespaceCollision, PragmaError
 from brownie.utils import color
@@ -21,28 +22,33 @@ class Sources:
         self._contracts: Dict = {}
         self._interfaces: Dict = {}
 
+        contracts: Dict = {}
         collisions: Dict = {}
         for path, source in contract_sources.items():
             self._source[path] = source
             if Path(path).suffix != ".sol":
-                contract_names = [Path(path).stem]
+                contract_names = [(Path(path).stem, "contract")]
             else:
                 contract_names = get_contract_names(source)
-            for name in contract_names:
-                if name in self._contracts:
-                    collisions.setdefault(name, set()).update([path, self._contracts[name]])
-                self._contracts[name] = path
+            for name, type_ in contract_names:
+                if name in contracts:
+                    if type_ == "interface":
+                        # allow names to overlap when dealing with interfaces
+                        continue
+                    if contracts[name][1] != "interface":
+                        collisions.setdefault(name, set()).update([path, contracts[name][0]])
+                contracts[name] = (path, type_)
+
+        self._contracts = {k: v[0] for k, v in contracts.items()}
 
         for path, source in interface_sources.items():
             self._source[path] = source
 
             if Path(path).suffix != ".sol":
-                interface_names = [Path(path).stem]
+                interface_names = [(Path(path).stem, "interface")]
             else:
                 interface_names = get_contract_names(source)
-            for name in interface_names:
-                if name in self._contracts:
-                    collisions.setdefault(name, set()).update([path, self._contracts[name]])
+            for name, type_ in interface_names:
                 if name in self._interfaces:
                     collisions.setdefault(name, set()).update([path, self._interfaces[name]])
                 self._interfaces[name] = path
@@ -163,23 +169,23 @@ def get_contract_names(full_source: str) -> List:
     Args:
         full_source: Solidity source code
 
-    Returns: list of contract names
+    Returns: list of (contract name, type)
     """
     # remove comments in case they contain code snippets that could fail the regex
     comment_regex = r"(?:\s*\/\/[^\n]*)|(?:\/\*[\s\S]*?\*\/)"
     uncommented_source = re.sub(comment_regex, "", full_source)
     contracts = re.findall(
-        r"((?:abstract contract|contract|library|interface)\s[^;{]*{[\s\S]*?})\s*(?=(?:abstract contract|contract|library|interface)\s|$)",  # NOQA: E501
+        r"((?:abstract contract|contract|library|interface)\s[^;{]*{[\s\S]*?})\s*(?=(?:abstract contract|contract|library|interface|pragma|struct|enum)\s|$)",  # NOQA: E501
         uncommented_source,
     )
 
     contract_names = []
     for source in contracts:
-        _, name, _ = re.findall(
+        type_, name, _ = re.findall(
             r"(abstract contract|contract|library|interface)\s+(\S*)\s*(?:is\s+([\s\S]*?)|)(?:{)",
             source,
         )[0]
-        contract_names.append(name)
+        contract_names.append((name, type_))
     return contract_names
 
 
@@ -203,3 +209,36 @@ def get_pragma_spec(source: str, path: Optional[str] = None) -> NpmSpec:
     if path:
         raise PragmaError(f"No version pragma in '{path}'")
     raise PragmaError("String does not contain a version pragma")
+
+
+def get_vyper_pragma_spec(source: str, path: Optional[str] = None) -> NpmSpec:
+    """
+    Extracts pragma information from Vyper source code.
+
+    Args:
+        source: Vyper source code
+        path: Optional path to the source (only used for error reporting)
+
+    Returns: NpmSpec object
+    """
+    pragma_match = next(re.finditer(r"(?:\n|^)\s*#\s*@version\s*([^\n]*)", source), None)
+    if pragma_match is None:
+        if path:
+            raise PragmaError(f"No version pragma in '{path}'")
+        raise PragmaError("String does not contain a version pragma")
+
+    pragma_string = pragma_match.groups()[0]
+    pragma_string = " ".join(pragma_string.split())
+    try:
+        return NpmSpec(pragma_string)
+    except ValueError:
+        pass
+    try:
+        # special case for Vyper 0.1.0-beta.X
+        version = to_vyper_version(pragma_string)
+        return NpmSpec(str(version))
+    except Exception:
+        pass
+
+    path = "" if path is None else f"{path}: "
+    raise PragmaError(f"{path}Cannot parse Vyper version from pragma: {pragma_string}")
